import sys
import os
from subprocess import call


# The path to the place where the tool is installed"
toolpath="/.statelite/tmpfs/gpfs/scratch/bsc32/bsc32870/RemoveLandProcesses"

# Patch to correct the path
toolpath+="/"
srcpath=toolpath+"/src/"
srcpath = os.path.relpath(srcpath, toolpath)
templatepath = srcpath+"/template.namelist"
print templatepath
assert os.path.isfile(templatepath), "Namelist template not found in src"
f = open(templatepath)

def Check_Parameters(parameters,resolutions):
   if len(parameters) != 3:
      print "Arguments needed:"
      print "       Resolution ( Options are : ORCA2, ORCA1, ORCA025, ORCA12)"
      print "       Number of processors"
      exit(1)
   
   if parameters[1] not in resolutions:
     print resolution, "ERROR: not a valid resolution"
     exit(1)


def primes(n):
    primfac = []
    d = 2
    while d*d <= n:
        while (n % d) == 0:
            primfac.append(d)  # supposing you want multiple factors repeated
            n //= d
        d += 1
    if n > 1:
       primfac.append(n)
    return primfac

def decompose(n):
   p = primes(n)
   while len(p) > 2:
       t = p[0]
       p = p[1:]
       p[-2] = p[-2]*t
       p.sort()
   p.reverse()
   if len(p) == 2:
      return p
   else:
      return p+[1]


#Resolution description
resolutions = ["ORCA2","ORCA1","ORCA025","ORCA12"]
grid_jpi =  {'ORCA2': '182', 'ORCA1': '362', 'ORCA025': '1442','ORCA12':'4322'}; 
grid_jpj =  {'ORCA2': '149', 'ORCA1': '292', 'ORCA025': '1021','ORCA12':'3059'}; 
grid_jpk =  {'ORCA2': '31', 'ORCA1': '75', 'ORCA025': '75','ORCA12':'75'}; 



Check_Parameters(sys.argv,resolutions)



resolution = sys.argv[1]
bathymetry_file = toolpath+"/bathymetries/bathy_meter_"+resolution+".nc" 
bathymetry_file = os.path.relpath(bathymetry_file, toolpath)
numprocs = int(sys.argv[2])
jpi = grid_jpi[resolution]
jpj = grid_jpj[resolution]
jpk = grid_jpk[resolution]
text = f.read()
text = text.replace("%JPI%",jpi)
text = text.replace("%JPJ%",jpj)
text = text.replace("%JPK%",jpk)
text = text.replace("%RESOLUTION%",resolution)
text = text.replace("%BATHYMETRY_FILE%",bathymetry_file)
original_text = text[:]

print "Resolution ",resolution
print "Max number of processors:", numprocs

#For valid cases
cases = []
for n in range(int(numprocs*0.8), int(numprocs*1.3)):
   jpni,jpnj = decompose(n)
   if 5 > float(jpni)/float(jpnj) > 0.2:
      text = original_text[:]
      text = text.replace("%JPNI%",str(jpni))
      text = text.replace("%JPNJ%",str(jpnj))
      out = open("namelist","w")
      out.write(text)
      out.close()
      stdout=open(srcpath+"out","w")
      stderr=open(srcpath+"err","w")
      status = call([toolpath+"/bin/find_sea_procs.exe","2&> /dev/null "],stdout=stdout,stderr=stderr)
      if status:
         print "ERROR num %i" % status
      stdout.close()
      stderr.close()
      stderr=open(srcpath+"err","r")
      error_log = stderr.read()
      stderr.close()
      if error_log.find("shared libraries") > -1:
         print "ERROR:"
         print "Load proper environment"
         print 
         exit(1)
      
      
      with open("./fort.8") as inp:
       for line in inp:
          if line.count("nombre de processeurs mer"):
            procs = int(line.split()[-1])
          if line.count("% sup"):
            coef = float(line.split()[-1])
      if procs <= int(numprocs):
         if coef == 0:
            coef = 10
         cases.append([jpni, jpnj ,n,procs, coef, procs/coef**2])

cases = sorted(cases,key=lambda x: x[-1])
print 
print 
print "It will be better if you run with one of these configurations:"
print "Bigger coefficient is better!"
if len(cases) > 5:
   cases = cases[-5:]
for case in list(reversed(cases)):
 # print "jpi:",case[0],"  jpj:",case[1], " Number of processors:", case[3], "Coeficient:",case[-1]
 jpi = case[0]
 jpj = case[1]
 OriginalP = jpi*jpj
 SeaP = case[3]
 LandP = OriginalP - SeaP
 Coef = case[-1]
 print "jpij: %i jpi: %i jpj: %i  OriginalP: %i  LandP: %i SeaP: %i  Coeficient: %.2f" % ( SeaP, jpi, jpj, OriginalP, LandP, SeaP, Coef)


files = os.listdir(".")
for f in files:
   if f.find(resolution+"-") ==0 and f.count("x") and f.count("_") and not f.count("."):
      os.remove(f)

os.remove("fort.8")
os.remove(srcpath+"err")
os.remove(srcpath+"out")
os.remove(toolpath+"namelist")
